.export loadframe
.export CLEARGRAPH
.export SETAUDIOREAD

VERA_LADDR := $9F20
VERA_MADDR := $9F21
VERA_HADDR := $9F22
VERA_DATA0 := $9F23
VERA_DATA1 := $9F24
 VERA_CTRL := $9F25
 VERA_ISR  := $9F27
AUDIO_CTRL := $9F3B
AUDIO_FIFO := $9F3D
AUDIO_RATE := $9F3C

CHKIN := $FFC6
CLRCHN := $FFCC
MACPTR := $FF44
GETIN := $FFE4
READST := $FFB7
kbdbuf_peek := $FEBD


VERA_RATE = $32
VERA_CFG = $33
PAL_ADDR1 = $34
PAL_ADDR2 = $35
loop_ctr2 := $66
loop_ctr1 := $67
PlayDone = $70
FramesDone = $71
R0L=2
R0H=3
R1L=4
R1H=5
R2L=6
R2H=7
R3L=8
R3H=9
R4L=$0A
R4H=$0B
R5L=$0C
R5H=$0D
R6L=$0E
R6H=$0F

R14L=$1E
R14H=$1F
R15L=$20
palptr = $26

.org $A000
jmp startplay                ;SYS $A000   (Starts Video Play)
jmp CLEARGRAPH               ;SYS $A003   (Utility routine, clears the GRAPHICS Buffer)
jmp SETAUDIOREAD             ;SYS $A006   (Sets the Size of the Audio Frame read)
jmp SETSPRITEREAD_8BPP       ;SYS $A009   (Sets up for 8 bpp (with palette) playback
jmp SETSPRITEREAD_4BPP       ;SYS $A00C   (Sets up for 4 bpp (with palette) playback

startplay:
   lda R0L
   sta VERA_RATE
   lda R0H
   ora #%00001111     ; for now set FULL VOLUME
   sta VERA_CFG
 lda 1
 pha
 stz 1
   stz PlayDone
   stz FramesDone
   stz AUDIO_RATE
   lda #%10000000
   sta AUDIO_CTRL    ; reset the Audio FIFO
   lda VERA_CFG
   sta AUDIO_CTRL
abuffpreloadloops:
   lda #2
   sta loop_ctr2

        ldx #6
        jsr CHKIN
aloop2: lda #6
        sta loop_ctr1
aloop1: ldx #<AUDIO_FIFO
        ldy #>AUDIO_FIFO
        lda #255
        sec
        jsr MACPTR
        dec loop_ctr1
        bne aloop1
        ldx #<AUDIO_FIFO
        ldy #>AUDIO_FIFO
abuffpreloadrem:
        lda #114
        sec
        jsr MACPTR
        dec loop_ctr2
        bne aloop2
        lda #6
        sta loop_ctr1
        jsr CLRCHN
        stz R15L
        lda VERA_RATE
        sta AUDIO_RATE    ; start play after filling the FIFO
loadframe_call1:
        jsr loadframe

ChkPlay:  lda PlayDone
          cmp #0
          bne ChkFrames
          ldx #6
          jsr CHKIN

waitaudio:
          lda VERA_ISR
          and #$08
          beq waitaudio
          ldx #<AUDIO_FIFO
          ldy #>AUDIO_FIFO
          lda #255
          sec
          jsr MACPTR
          ldx #<AUDIO_FIFO
          ldy #>AUDIO_FIFO
          lda #255
          sec
          jsr MACPTR
frmstrt:  lda PlayDone
          cmp #0
          bne ChkFrames
          lda VERA_RATE
          sta AUDIO_RATE
abuff255: lda #4
          sta loop_ctr1
aloop3:   ldx #<AUDIO_FIFO
          ldy #>AUDIO_FIFO
          lda #255
          sec
          jsr MACPTR
          dec loop_ctr1
          bne aloop3
          ldx #<AUDIO_FIFO
          ldy #>AUDIO_FIFO
abuffrem: lda #114
          sec
          jsr MACPTR
          jsr READST
          bne SetPlayDone
          jsr kbdbuf_peek
          bne checkkeys
ChkFrames:
          lda FramesDone
          cmp #0
          bne ChkAudioDone
loadframe_call2:
          jsr loadframe
          jsr READST
          bne SetFrmsDone
          bra ChkPlay

SetPlayDone:
  lda #$FF
  sta PlayDone
  bra ChkFrames

checkkeys:
  stz AUDIO_RATE
  jsr CLRCHN
  jsr GETIN
donekeys:
  cmp #27
  beq setdone
  cmp #32
  beq dopause
  lda VERA_RATE
  sta AUDIO_RATE
  bra ChkFrames

setdone:
  lda #$FF
  sta FramesDone
  sta PlayDone
  bra ChkFrames

SetFrmsDone:
  lda #$FF
  sta FramesDone
  jmp ChkPlay

ChkAudioDone:
  lda PlayDone
  cmp #0
  bne over 
  jmp ChkPlay
over:
         jsr CLRCHN
  pla
  sta 1
         rts

dopause:
  jsr GETIN
  cmp #27
  beq donekeys
  cmp #32
  beq endpause
  bra dopause
endpause:
  lda #0
  bra donekeys

loadframe:
    ldx #4
	jsr CHKIN
SBuffSize255:
    lda #70
    sta R0L
    lda R15L
    beq Hipage
SpriteAttrEndAddr:
    ldx #$36
    ldy #$FE         ; start at sprite 70 (unless modified)
    lda #%01001001   ; and decrement
    jsr swapsprites

    stz R15L
    stz VERA_LADDR
    stz VERA_MADDR
    bra Hiaddr

Hipage:
    ldx #$0E
    ldy #$FC
    lda #%01000001
    jsr swapsprites

    lda #1
    sta R15L
    stz VERA_LADDR
HiSpriteDataMidAddr:
    lda #$48
    sta VERA_MADDR
Hiaddr:
    lda #%00010000
    sta VERA_HADDR
loop1:
	ldx #$23
	ldy #$9F
	lda #255
	sec
	jsr MACPTR
	dec R0L
	bne loop1
	ldx #$23
	ldy #$9F
SBuffSizeRemainder:
	lda #70
	sec
	jsr MACPTR

; now load the palette for the frame
    lda VERA_LADDR
    sta PAL_ADDR1
    lda VERA_MADDR
    sta PAL_ADDR2
	ldx #$23
	ldy #$9F
	lda #255
	sec
	jsr MACPTR
	ldx #$23
	ldy #$9F
	lda #255
	sec
	jsr MACPTR
	ldx #$23
	ldy #$9F
	lda #2
	sec
    jmp MACPTR
; tail call optimization
; MACPTR's rts works for loadframes sub



swapsprites:
          stx VERA_LADDR
          sty VERA_MADDR
          sta VERA_HADDR
          lda #1
          sta VERA_CTRL
          lda PAL_ADDR1
          sta VERA_LADDR
          lda PAL_ADDR2
          sta VERA_MADDR
          lda #%00010000
          sta VERA_HADDR
          stz VERA_CTRL
SpriteCount:
          ldx #35
          ldy #35
          lda #%00001100
          wai
sprloop1: sta VERA_DATA0
          dex
          bne sprloop1
sprloop2: stz VERA_DATA0
          dey
          bne sprloop2

          stz VERA_LADDR
          lda #$FA       ; THE PALETTE
          sta VERA_MADDR
          lda #%00010001
          sta VERA_HADDR

ploop2:   ldx #32
ploop1:   lda VERA_DATA1
          sta VERA_DATA0

          lda VERA_DATA1
          sta VERA_DATA0

          lda VERA_DATA1
          sta VERA_DATA0

          lda VERA_DATA1
          sta VERA_DATA0

          lda VERA_DATA1
          sta VERA_DATA0

          lda VERA_DATA1
          sta VERA_DATA0

          lda VERA_DATA1
          sta VERA_DATA0

          lda VERA_DATA1
          sta VERA_DATA0

          lda VERA_DATA1
          sta VERA_DATA0

          lda VERA_DATA1
          sta VERA_DATA0

          lda VERA_DATA1
          sta VERA_DATA0

          lda VERA_DATA1
          sta VERA_DATA0

          lda VERA_DATA1
          sta VERA_DATA0

          lda VERA_DATA1
          sta VERA_DATA0

          lda VERA_DATA1
          sta VERA_DATA0

          lda VERA_DATA1
          sta VERA_DATA0

          dex
          bne ploop1
          rts

loadframe_4bpp:
    ldx #4
	jsr CHKIN
SBuffSize255_4bpp:
    lda #35
    sta R0L
    lda R15L
    beq Hipage_4bpp

SpriteAttrEndAddr_4bpp:
   ldx #$36
   ldy #$FE         ; start at sprite 70
   lda #%01001001   ; and decrement
   jsr swapsprites_4bpp
   ldx #$C0
   stx palptr

    stz R15L
    stz VERA_LADDR
    lda #$30
    sta VERA_MADDR
    bra Hiaddr_4bpp

Hipage_4bpp:
    ldx #$0E
    ldy #$FC
    lda #%01000001
    jsr swapsprites_4bpp
    ldx #$E0
    stx palptr

    lda #1
    sta R15L
    stz VERA_LADDR
HiSpriteDataMidAddr_4bpp:
    lda #$53
    sta VERA_MADDR
Hiaddr_4bpp:
    lda #%00010001
    sta VERA_HADDR
loop1_4bpp:
	ldx #$23
	ldy #$9F
	lda #$FF
	sec
	jsr MACPTR
	dec R0L
	bne loop1_4bpp
	ldx #$23
	ldy #$9F
SBuffSizeRemainder_4bpp:
	lda #35
	sec
	jsr MACPTR

; NOW WE GET THE PALETTE DATA FOR THE FRAME
    lda palptr
	sta VERA_LADDR
	lda #$FB
	sta VERA_MADDR
	lda #%00010001
	sta VERA_HADDR
	ldx #$23
	ldy #$9F
	lda #32
	sec
	jmp MACPTR
; tail call optimization JMP instead of JSR/RTS


swapsprites_4bpp:
          wai
          stx VERA_LADDR
          sty VERA_MADDR
          sta VERA_HADDR
SpriteCount4bpp:
          ldx #35
          ldy #35
          lda #%00001100
sprloop1_4bpp:
          sta VERA_DATA0
          dex
          bne sprloop1_4bpp
sprloop2_4bpp:
          stz VERA_DATA0
          dey
          bne sprloop2_4bpp
          lda palptr
          sta VERA_LADDR
          lda #$FB
          sta VERA_MADDR
          lda #%00010001
          sta VERA_HADDR
          lda VERA_DATA0
          sta palbytes
          lda VERA_DATA0
          sta palbytes+1
          stz VERA_LADDR
          lda #$FA
          sta VERA_MADDR
          lda palbytes
          sta VERA_DATA0
          lda palbytes+1
          sta VERA_DATA0
          rts

savectrl: .byte 0
palbytes: .word 00

CLEARGRAPH:
        stz $9F20
        stz $9F21
        lda #%00010000
        sta $9F22
        lda #0
        ldy #2
Cloop2:  ldx #150
Cloop1:  stz $9F23
        ina
        bne Cloop1
        dex
        bne Cloop1
        dey
        bne Cloop2
        rts

SETAUDIOREAD:
        lda R0L
        sta abuffpreloadloops+1
        lda R0H
        sta aloop2+1
        lda R1L
        sta abuffpreloadrem+1
        lda R1H
        dea
        dea
        sta abuff255+1
        lda R2L
        sta abuffrem+1
        rts

SETSPRITEREAD_8BPP:
        lda R3L
        sta SBuffSize255+1
        lda R3H
        sta SBuffSizeRemainder+1
        lda R4L
        sta SpriteCount+1
        sta SpriteCount+3
        lda R5L
        sta SpriteAttrEndAddr+1
        lda R5H
        sta SpriteAttrEndAddr+3
        lda R6H
        sta HiSpriteDataMidAddr+1
        lda #<loadframe
        ldx #>loadframe
        sta loadframe_call1+1
        sta loadframe_call2+1
        stx loadframe_call1+2
        stx loadframe_call2+2
        rts

SETSPRITEREAD_4BPP:
        lda R3L
        sta SBuffSize255_4bpp+1
        lda R3H
        sta SBuffSizeRemainder_4bpp+1
        lda R4L
        sta SpriteCount4bpp +1
        sta SpriteCount4bpp+3
        lda R5L
        sta SpriteAttrEndAddr_4bpp+1
        lda R5H
        sta SpriteAttrEndAddr_4bpp+3
        lda R6H
        sta HiSpriteDataMidAddr_4bpp +1
        lda #<loadframe_4bpp
        ldx #>loadframe_4bpp
        sta loadframe_call1+1
        sta loadframe_call2+1
        stx loadframe_call1+2
        stx loadframe_call2+2
        rts



